@@ -5,10 +5,13 @@ from django.template.engine import Engine
 from django.template.loader import render_to_string
 from django.utils.html import format_html, mark_safe
 
-from debug_toolbar.decorators import require_show_toolbar
+from debug_toolbar._compat import login_not_required
+from debug_toolbar.decorators import render_with_toolbar_language, require_show_toolbar
 
 
+@login_not_required
 @require_show_toolbar
+@render_with_toolbar_language
 def template_source(request):
     """
     Return the source of a template, syntax-highlighted by Pygments if
@@ -53,7 +56,7 @@ def template_source(request):
     except ModuleNotFoundError:
         source = format_html("<code>{}</code>", source)
     else:
-        source = highlight(source, HtmlDjangoLexer(), HtmlFormatter())
+        source = highlight(source, HtmlDjangoLexer(), HtmlFormatter(wrapcode=True))
         source = mark_safe(source)
 
     content = render_to_string(
