@@ -5,11 +5,12 @@ The main DebugToolbar class that loads and renders the Toolbar.
 import re
 import uuid
 from collections import OrderedDict
-from functools import lru_cache
+from functools import cache
 
 from django.apps import apps
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
+from django.core.handlers.asgi import ASGIRequest
 from django.dispatch import Signal
 from django.template import TemplateSyntaxError
 from django.template.loader import render_to_string
@@ -19,6 +20,7 @@ from django.utils.module_loading import import_string
 from django.utils.translation import get_language, override as lang_override
 
 from debug_toolbar import APP_NAME, settings as dt_settings
+from debug_toolbar.panels import Panel
 
 
 class DebugToolbar:
@@ -38,7 +40,7 @@ class DebugToolbar:
         # Use OrderedDict for the _panels attribute so that items can be efficiently
         # removed using FIFO order in the DebugToolbar.store() method.  The .popitem()
         # method of Python's built-in dict only supports LIFO removal.
-        self._panels = OrderedDict()
+        self._panels = OrderedDict[str, Panel]()
         while panels:
             panel = panels.pop()
             self._panels[panel.panel_id] = panel
@@ -98,12 +100,18 @@ class DebugToolbar:
         If False, the panels will be loaded via Ajax.
         """
         if (render_panels := self.config["RENDER_PANELS"]) is None:
-            # If wsgi.multiprocess isn't in the headers, then it's likely
-            # being served by ASGI. This type of set up is most likely
-            # incompatible with the toolbar until
-            # https://github.com/jazzband/django-debug-toolbar/issues/1430
-            # is resolved.
-            render_panels = self.request.META.get("wsgi.multiprocess", True)
+            # If wsgi.multiprocess is true then it is either being served
+            # from ASGI or multithreaded third-party WSGI server eg gunicorn.
+            # we need to make special check for ASGI for supporting
+            # async context based requests.
+            if isinstance(self.request, ASGIRequest):
+                render_panels = False
+            else:
+                # The wsgi.multiprocess case of being True isn't supported until the
+                # toolbar has resolved the following issue:
+                # This type of set up is most likely
+                # https://github.com/django-commons/django-debug-toolbar/issues/1430
+                render_panels = self.request.META.get("wsgi.multiprocess", True)
         return render_panels
 
     # Handle storing toolbars in memory and fetching them later on
@@ -165,14 +173,14 @@ class DebugToolbar:
         # not have resolver_match set.
         try:
             resolver_match = request.resolver_match or resolve(
-                request.path, getattr(request, "urlconf", None)
+                request.path_info, getattr(request, "urlconf", None)
             )
         except Resolver404:
             return False
         return resolver_match.namespaces and resolver_match.namespaces[-1] == APP_NAME
 
     @staticmethod
-    @lru_cache(maxsize=None)
+    @cache
     def get_observe_request():
         # If OBSERVE_REQUEST_CALLBACK is a string, which is the recommended
         # setup, resolve it to the corresponding callable.
@@ -208,7 +216,7 @@ def debug_toolbar_urls(prefix="__debug__"):
         return []
     return [
         re_path(
-            r"^%s/" % re.escape(prefix.lstrip("/")),
+            r"^{}/".format(re.escape(prefix.lstrip("/"))),
             include("debug_toolbar.urls"),
         ),
     ]
